﻿using Microsoft.AspNetCore.Mvc;
using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.BusinessLogicsContracts;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.ViewModels;

namespace PrecastConcretePlantRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ShopController : Controller
    {
        private readonly ILogger _logger;

        private readonly IShopLogic _logic;

        public ShopController(IShopLogic logic, ILogger<ShopController> logger)
        {
            _logger = logger;
            _logic = logic;
        }

        [HttpGet]
        public List<ShopViewModel>? GetShopList()
        {
            try
            {
                return _logic.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Receiving shop list error");
                throw;
            }
        }

        [HttpGet]
        public ShopViewModel? GetShop(int shopId)
        {
            try
            {
                var shop = _logic.ReadElement(new ShopSearchModel { Id = shopId });
                if (shop != null)
                {
                    shop.ShortReinforcedInfo = shop.ShopReinforceds.Select(x => Tuple.Create(x.Value.Item1.ReinforcedName, x.Value.Item1.Price, x.Value.Item2)).ToList();
                }
                return shop;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error receiving shop by id = {Id}", shopId);
                throw;
            }
        }

        [HttpPost]
        public void CreateShop(ShopBindingModel model)
        {
            try
            {
                _logic.Create(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Shop creation error");
                throw;
            }
        }

        [HttpPost]
        public void UpdateShop(ShopBindingModel model)
        {
            try
            {
                _logic.Update(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Shop update error");
                throw;
            }
        }

        [HttpPost]
        public void DeleteShop(ShopBindingModel model)
        {
            try
            {
                _logic.Delete(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Shop deletion error");
                throw;
            }
        }

        [HttpPost]
        public void ReplenishShop(Tuple<ShopSearchModel, ReinforcedViewModel, int> shopWorks)
        {
            try
            {
                _logic.ReplenishShop(shopWorks.Item1, shopWorks.Item2, shopWorks.Item3);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Replenishment error");
                throw;
            }
        }
    }
}
