﻿using Microsoft.AspNetCore.Mvc;
using System.Diagnostics;
using System.Net;
using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantShopApp.Models;
using PrecastConcretePlantShopApp;

namespace FoodOrdersShopApp.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;

        public HomeController(ILogger<HomeController> logger)
        {
            _logger = logger;
        }

        public IActionResult Index()
        {
            if (!APIClient.IsAuthenticated)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<ShopViewModel>>($"api/shop/getshoplist"));
        }

        [HttpGet]
        public IActionResult Enter()
        {
            return View();
        }

        [HttpPost]
        public void Enter(string password)
        {
            if (string.IsNullOrEmpty(password))
            {
                throw new Exception("Введите пароль");
            }
            if (!APIClient.Authentication(password))
            {
                throw new Exception("Неверный пароль");
            }
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Create()
        {
            if (!APIClient.IsAuthenticated)
            {
                return Redirect("~/Home/Enter");
            }
            return View("Shop");
        }

        [HttpPost]
        public void Create(string shopName, string address, DateTime dateOpening, int reinforcedsMax)
        {
            if (string.IsNullOrEmpty(shopName))
            {
                throw new Exception("Название магазина не указано");
            }
            if (string.IsNullOrEmpty(address))
            {
                throw new Exception("Адрес магазина не указан");
            }
            if (reinforcedsMax <= 0)
            {
                throw new Exception("Вместимость магазина должна быть больше нуля");
            }
            APIClient.PostRequest("api/shop/createshop", new ShopBindingModel
            {
                ShopName = shopName,
                Address = address,
                DateOpening = dateOpening,
                ReinforcedsMax = reinforcedsMax
            });
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Update(int id)
        {
            if (!APIClient.IsAuthenticated)
            {
                return Redirect("~/Home/Enter");
            }
            return View("Shop", APIClient.GetRequest<ShopViewModel>($"api/shop/getshop?shopId={id}"));
        }

        [HttpPost]
        public void Update(int id, string shopName, string address, DateTime dateOpening, int reinforcedsMax)
        {
            if (string.IsNullOrEmpty(shopName))
            {
                throw new Exception("Название магазина не указано");
            }
            if (string.IsNullOrEmpty(address))
            {
                throw new Exception("Адрес магазина не указан");
            }
            if (reinforcedsMax <= 0)
            {
                throw new Exception("Вместимость магазина должна быть больше нуля");
            }
            APIClient.PostRequest($"api/shop/updateshop", new ShopBindingModel
            {
                Id = id,
                ShopName = shopName,
                Address = address,
                DateOpening = dateOpening,
                ReinforcedsMax = reinforcedsMax
            });
            Response.Redirect("../Index");
        }

        [HttpPost]
        public void Delete(int id)
        {
            APIClient.PostRequest($"api/shop/deleteshop", new ShopBindingModel
            {
                Id = id,
            });
            Response.Redirect("../Index");
        }

        [HttpGet]
        public IActionResult ReplenishShop()
        {
            if (!APIClient.IsAuthenticated)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Shops = APIClient.GetRequest<List<ShopViewModel>>($"api/shop/getshoplist");
            ViewBag.Reinforceds = APIClient.GetRequest<List<ReinforcedViewModel>>($"api/main/getreinforcedlist");
            return View();
        }

        [HttpPost]
        public void ReplenishShop(int shop, int reinforced, int count)
        {
            if (count <= 0)
            {
                throw new Exception("Количество изделий в поставке должно быть больше 0");
            }
            APIClient.PostRequest("api/shop/replenishShop", Tuple.Create(
                new ShopSearchModel() { Id = shop },
                new ReinforcedViewModel() { Id = reinforced },
                count
            ));
            Response.Redirect("Index");
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}